%%  Some analyses of the SweSAT quantitative data

%  number of items

n = 80;

%  number of values over [0,n] for plotting purposes

nfine   = 101;
scrfine = linspace(0.1,n-0.1,nfine)';
scrrng  = [0.1,n-0.1];

%  load an fd object containing the 80 the W functions for the 
%  optimal analysis

load Wfd_opt

%   set up WfdCell for B-spline population W functions

WfdCell_opt = cell(n,1);
for i=1:n
    WStri.Wfd = Wfd_opt(i);
    WStri.M   = 1;
    WfdCell_opt{i} = WStri;
end

%  load an fd object containing the 80 the W functions for the 3PL analysis

load Wfd_3PL

%   set up WfdCell for 3PL population W functions

WfdCell_3PL = cell(n,1);
for i=1:n
    WStri.Wfd = Wfd_3PL(i);
    WStri.M   = 1;
    WfdCell_3PL{i} = WStri;
end

%  load the log density function fd object for the distribution of theta's

load logdensfdobj

%  load the cumulative density function for the distribution of theta's

load thetadenscdf_opt

%%  Simulate 1000 samples with theta at 0:n

parpool local

nsample = 1000;

Usave      = zeros(nfine,n,nsample);
Wmat_sim   = eval_fd(scrfine, Wfd_opt);
Emat_sim   = exp(Wmat_sim);
Pmat_sim   = Emat_sim./(1+Emat_sim);
expscr_sim = sum(Pmat_sim,2);
for isample=1:nsample
    Ui = zeros(nfine,n);
    for j = 1:nfine
        Uj = zeros(1,n);
        Uj(Pmat_sim(j,:) > rand(1,n)) = 1;
        Ui(j,:) = Uj;
    end
    Usave(:,:,isample) = Ui;
end

%%  Analyze each sample

%  We set Slambda to 0 here because the distribution of theta is already
%  by working with pre-calculated W functions.
Slambda = 0;
sumscrsave    = zeros(nfine, nsample);
thetasave_opt = zeros(nfine, nsample);
thetasave_3PL = zeros(nfine, nsample);
%  Here we are not bothering with computed the expected sum score
% expscrsave    = zeros(nfine, nsample);
tic;
parfor isample=1:nsample
    if mod(isample,100) == 0
        disp(['sample ',num2str(isample)])
    end
    Ui = squeeze(Usave(:,:,isample));
    %  sum scores for this sample
    scrveci = sum(Ui,2);
    sumscrsave(:,isample) = scrveci;
    %  optimal scores for this sample
    thetai_opt  = thetafun_sim(scrveci, WfdCell_opt, Ui, scrrng, Slambda);
    thetasave_opt(:,isample)  = thetai_opt;
    %  3PL scores for this sample
    thetai_3PL  = thetafun_sim(scrveci, WfdCell_3PL, Ui, scrrng, Slambda);
    thetasave_3PL(:,isample)  = thetai_3PL;
%     %  optimal expected score estimates for this sample
%     Wmati = eval_fd(thetai, Wfd_opt);
%     Emati = exp(Wmati);
%     Pmati = Emati./(1 + Emati);
%     expscri = sum(Pmati,2);
%     expscrsave(:,isample) = expscri;
end
toc

%  compute bias and RMSE

thetares_opt = thetasave_opt  - scrfine*ones(1,nsample);
thetares_3PL = thetasave_3PL  - scrfine*ones(1,nsample);
sumscrres    = sumscrsave - scrfine*ones(1,nsample);
% expscrres    = expscrsave - scrfine*ones(1,nsample);

%  get bias's averaged over both samples and theta's

thetabias_opt = mean(mean(thetares_opt));
thetabias_3PL = mean(mean(thetares_3PL));
sumscrbias    = mean(mean(sumscrres));
% expscrbias    = mean(mean(expscrres));

%  get RMSE's averaged over both samples and theta's

thetaRMSE_opt = sqrt(mean(mean(thetares_opt.^2)));
thetaRMSE_3PL = sqrt(mean(mean(thetares_3PL.^2)));
sumscrRMSE    = sqrt(mean(mean(sumscrres.^2)));
% expscrRMSE    = sqrt(mean(mean(expscrres.^2)));

%  report doubly averaged results

disp(' ')
disp('RMSE results for theta estimates:')
disp(['Optimal theta score  RMSE = ',num2str(thetaRMSE_opt)])
disp(['3PL     theta score  RMSE = ',num2str(thetaRMSE_3PL)])
disp(['Sum  score  RMSE = ',num2str(sumscrRMSE)])
% disp(['Exp. score  RMSE = ', num2str(expscrRMSE)])

disp(' ')
disp('Bias results for theta estimates:')
disp(['Optimal theta score  bias = ', num2str(thetabias_opt)])
disp(['3PL     theta score  bias = ', num2str(thetabias_3PL)])
disp(['Sum   score  bias = ', num2str(sumscrbias)])
% disp(['Exp. score  bias = '  ,num2str(expscrbias)])

%  compute bias's averaged over only samples

thetabias2_opt = mean(thetares_opt, 2);
thetabias2_3PL = mean(thetares_3PL, 2);
sumscrbias2    = mean(sumscrres, 2);
% expscrbias2    = mean(expscrres, 2);

%  compute RMSE's averaged over only samples

thetaRMSE2_opt = sqrt(mean(thetares_opt.^2, 2));
thetaRMSE2_3PL = sqrt(mean(thetares_3PL.^2, 2));
sumscrRMSE2    = sqrt(mean(sumscrres.^2, 2));
% expscrRMSE2    = sqrt(mean(expscrres.^2, 2));

%  Lightly smooth bias results over theta's

biasbasis = create_bspline_basis([0,n], 39);
biasfdPar = fdPar(biasbasis, 2, 1e0);

thetabias2fd_opt  = smooth_basis(scrfine, thetabias2_opt,  biasfdPar);
thetabias2fd_3PL  = smooth_basis(scrfine, thetabias2_3PL,  biasfdPar);
sumscrbias2fd     = smooth_basis(scrfine, sumscrbias2, biasfdPar);
% expscrbias2fd     = smooth_basis(scrfine, expscrbias2, biasfdPar);
thetabias2vec_opt = eval_fd(scrfine, thetabias2fd_opt);
thetabias2vec_3PL = eval_fd(scrfine, thetabias2fd_3PL);
sumscrbias2vec    = eval_fd(scrfine, sumscrbias2fd);
% expscrbias2vec    = eval_fd(scrfine, expscrbias2fd);

%  Lightly smooth RMSE results over theta's

RMSEbasis = create_bspline_basis([0,n], 39);
RMSEfdPar = fdPar(RMSEbasis, 2, 1e0);

thetaRMSE2fd_opt  = smooth_basis(scrfine, thetaRMSE2_opt,  RMSEfdPar);
thetaRMSE2fd_3PL  = smooth_basis(scrfine, thetaRMSE2_3PL,  RMSEfdPar);
sumscrRMSE2fd     = smooth_basis(scrfine, sumscrRMSE2, RMSEfdPar);
% expscrRMSE2fd     = smooth_basis(scrfine, expscrRMSE2, RMSEfdPar);
thetaRMSE2vec_opt = eval_fd(scrfine, thetaRMSE2fd_opt);
thetaRMSE2vec_3PL = eval_fd(scrfine, thetaRMSE2fd_3PL);
sumscrRMSE2vec    = eval_fd(scrfine, sumscrRMSE2fd);
% expscrRMSE2vec    = eval_fd(scrfine, expscrRMSE2fd);

%%  plot results

pvec0         = [0.0500    0.2500    0.5000    0.7500    0.9500];
thetaQvec_opt = [19.4208   27.4481   35.1728   45.0106   59.3866];

%  two panel result with RMSE and bias (bias displayed first)

figure(3)
%  ------------  bias results  ------------
subplot(2,1,1)
phdl = plot(scrfine, sumscrbias2vec,     'b--', ...
            scrfine, thetabias2vec_opt,  'b-', ...
            scrfine, thetabias2vec_3PL,  'b:', ...
            [0,n], [0,0], 'b--');
set(phdl, 'LineWidth', 2)
hold on
for k=1:length(pvec0)
    phdl = plot([thetaQvec_opt(k),thetaQvec_opt(k)], [-5,5], 'b--');
    set(phdl, 'LineWidth', 2)
end
hold off
axis([0,n,-5,5])
ylabel('\fontsize{24} Bias')
legend('\fontsize{16} Sumscore', ...
       '\fontsize{16} Optimal', ...
       '\fontsize{16} 3PL', ...
       'Location', 'South')
%  ------------  RMSE results  ------------
subplot(2,1,2)
phdl = plot(scrfine, sumscrRMSE2vec,     'b--', ...
            scrfine, thetaRMSE2vec_opt,  'b-', ...
            scrfine, thetaRMSE2vec_3PL,  'b:');
set(phdl, 'LineWidth', 2)
hold on
for k=1:5
    phdl = plot([thetaQvec_opt(k),thetaQvec_opt(k)], [2,6], 'b--');
    set(phdl, 'LineWidth', 2)
end
hold off
axis([0,n,2,6])
ylabel('\fontsize{24} RMSE')
xlabel('\fontsize{24} Score')

%  two panel result with RMSE and bias (RMSE first)

figure(4)
%  ------------  RMSE results  ------------
subplot(2,1,1)
phdl = plot(scrfine, sumscrRMSE2vec,     'b--', ...
            scrfine, thetaRMSE2vec_opt,  'b-', ...
            scrfine, thetaRMSE2vec_3PL,  'b:');
set(phdl, 'LineWidth', 2)
hold on
for k=1:length(pvec0)
    phdl = plot([thetaQvec_opt(k),thetaQvec_opt(k)], [0,7], 'b--');
    set(phdl, 'LineWidth', 2)
end
hold off
axis([0,n,0,7])
ylabel('\fontsize{24} Mean RMSE(\theta)')
legend('\fontsize{16} Sumscore', ...
       '\fontsize{16} Optimal', ...
       '\fontsize{16} 3PL', ...
       'Location', 'South')
%  ------------  bias results  ------------
subplot(2,1,2)
phdl = plot(scrfine, sumscrbias2vec,     'b--', ...
            scrfine, thetabias2vec_opt,  'b-', ...
            scrfine, thetabias2vec_3PL,  'b:', ...
            [0,n], [0,0], 'b--');
set(phdl, 'LineWidth', 2)
hold on
for k=1:length(pvec0)
    phdl = plot([thetaQvec_opt(k),thetaQvec_opt(k)], [-7,7], 'b--');
    set(phdl, 'LineWidth', 2)
end
hold off
axis([0,n,-8,8])
xlabel('\fontsize{24} \theta')
ylabel('\fontsize{24} Mean bias(\theta)')
